@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014-2019 <tsujan2000@gmail.com>
  *
  * Kvantum is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -22,6 +22,7 @@
 #include <QMap>
 #include <QItemDelegate>
 #include <QAbstractItemView>
+#include <QToolButton>
 
 #include "shortcuthandler.h"
 #include "drag/windowmanager.h"
@@ -31,6 +32,10 @@
 #include "animation/animation.h"
 #endif
 
+// definitions shared by source files
+#define SLIDER_TICK_SIZE 5 // 10 at most
+#define ANIMATION_FRAME 40 // in ms
+
 class QSvgRenderer;
 
 namespace Kvantum {
@@ -96,54 +101,61 @@ class Style : public QCommonStyle {
 
     virtual bool eventFilter(QObject *o, QEvent *e);
 
-    virtual int pixelMetric(PixelMetric metric,
+    virtual int pixelMetric(QStyle::PixelMetric metric,
                             const QStyleOption *option = 0,
                             const QWidget *widget = 0) const;
-    virtual QRect subElementRect(SubElement element,
+    virtual QRect subElementRect(QStyle::SubElement element,
                                  const QStyleOption *option,
                                  const QWidget *widget = 0) const;
-    virtual QRect subControlRect(ComplexControl control,
+    virtual QRect subControlRect(QStyle::ComplexControl control,
                                  const QStyleOptionComplex *option,
-                                 SubControl subControl,
+                                 QStyle::SubControl subControl,
                                  const QWidget *widget = 0) const;
-    QSize sizeFromContents(ContentsType type,
+    QSize sizeFromContents(QStyle::ContentsType type,
                            const QStyleOption *option,
                            const QSize &contentsSize,
                            const QWidget *widget = 0) const;
 
-    virtual void drawPrimitive(PrimitiveElement element,
+    virtual void drawPrimitive(QStyle::PrimitiveElement element,
                                const QStyleOption *option,
                                QPainter *painter,
                                const QWidget *widget = 0) const;
-    virtual void drawControl(ControlElement element,
+    virtual void drawControl(QStyle::ControlElement element,
                              const QStyleOption *option,
                              QPainter *painter,
                              const QWidget *widget = 0) const;
-    virtual void drawComplexControl(ComplexControl control,
+    virtual void drawComplexControl(QStyle::ComplexControl control,
                                     const QStyleOptionComplex *option,
                                     QPainter *painter,
                                     const QWidget *widget = 0) const;
-    virtual int styleHint(StyleHint hint,
+    virtual int styleHint(QStyle::StyleHint hint,
                           const QStyleOption *option = 0,
                           const QWidget *widget = 0,
                           QStyleHintReturn *returnData = 0) const;
-    virtual SubControl hitTestComplexControl(ComplexControl control,
-                                             const QStyleOptionComplex *option,
-                                             const QPoint &position,
-                                             const QWidget *widget = 0) const;
+    virtual QStyle::SubControl hitTestComplexControl(QStyle::ComplexControl control,
+                                                     const QStyleOptionComplex *option,
+                                                     const QPoint &position,
+                                                     const QWidget *widget = 0) const;
 
     virtual QPixmap generatedIconPixmap(QIcon::Mode iconMode,
                                         const QPixmap &pixmap,
                                         const QStyleOption *option) const;
 
+    virtual void drawItemText(QPainter *painter, const QRect &rect, int flags,
+                              const QPalette &pal, bool enabled, const QString &text,
+                              QPalette::ColorRole textRole = QPalette::NoRole) const;
+
+    virtual void drawItemPixmap(QPainter *painter, const QRect &rect,
+                                int alignment, const QPixmap &pixmap) const;
+
+    QIcon standardIcon(QStyle::StandardPixmap standardIcon,
+                       const QStyleOption *option = 0,
+                       const QWidget *widget = 0) const;
+
     enum CustomElements {
       CE_Kv_KCapacityBar = CE_CustomBase + 0x00FFFF00,
     };
 
-    QIcon standardIcon(StandardPixmap standardIcon,
-                       const QStyleOption *option = 0,
-                       const QWidget *widget = 0) const;
-
   private:
     /* For handling disabled icons with all icon engines: */
     enum KvIconMode {
@@ -186,15 +198,23 @@ class Style : public QCommonStyle {
     /* Return the state of the given widget. */
     QString getState(const QStyleOption *option, const QWidget *widget) const;
     /* Return the frame spec of the given widget from the theme config file. */
-    inline frame_spec getFrameSpec(const QString &widgetName) const;
+    frame_spec getFrameSpec(const QString &widgetName) const {
+      return settings_->getFrameSpec(widgetName);
+    }
     /* Return the interior spec of the given widget from the theme config file. */
-    inline interior_spec getInteriorSpec(const QString &widgetName) const;
+    interior_spec getInteriorSpec(const QString &widgetName) const {
+      return settings_->getInteriorSpec(widgetName);
+    }
     /* Return the indicator spec of the given widget from the theme config file. */
-    inline indicator_spec getIndicatorSpec(const QString &widgetName) const;
+    indicator_spec getIndicatorSpec(const QString &widgetName) const {
+      return settings_->getIndicatorSpec(widgetName);
+    }
     /* Return the label (text+icon) spec of the given widget from the theme config file. */
-    inline label_spec getLabelSpec(const QString &widgetName) const;
+    label_spec getLabelSpec(const QString &widgetName) const;
     /* Return the size spec of the given widget from the theme config file */
-    inline size_spec getSizeSpec(const QString &widgetName) const;
+    size_spec getSizeSpec(const QString &widgetName) const {
+      return settings_->getSizeSpec(widgetName);
+    }
 
     /* Generic method that draws a frame. */
     void renderFrame(QPainter *painter,
@@ -222,13 +242,14 @@ class Style : public QCommonStyle {
                        ) const;
 
     /* Generic method that draws an indicator. */
-    void renderIndicator(QPainter *painter,
+    bool renderIndicator(QPainter *painter,
                          const QRect &bounds, // frame bounds
                          const frame_spec &fspec, // frame spec
                          const indicator_spec &dspec, // indicator spec
                          const QString &element, // indicator SVG element
                          Qt::LayoutDirection ld = Qt::LeftToRight,
-                         Qt::Alignment alignment = Qt::AlignCenter) const;
+                         Qt::Alignment alignment = Qt::AlignCenter,
+                         int vOffset = 0) const;
 
     /* Generic method that draws a label (text and/or icon) inside the frame. */
     void renderLabel(
@@ -279,12 +300,8 @@ class Style : public QCommonStyle {
                          const size_spec &sspec,
                          const QString &text,
                          const QSize iconSize,
-                         const Qt::ToolButtonStyle tialign = Qt::ToolButtonTextBesideIcon, // text-icon alignment
-                         // use real heights of multiline texts?
-                         bool realHeight = false) const;
-
-    /* Return a normalized rect, i.e. a square. */
-    QRect squaredRect(const QRect &r) const;
+                         const Qt::ToolButtonStyle tialign = Qt::ToolButtonTextBesideIcon // text-icon alignment
+                        ) const;
 
     /* Return the remaining QRect after subtracting the frames. */
     QRect interiorRect(const QRect &bounds, const frame_spec &fspec) const;
@@ -293,6 +310,9 @@ class Style : public QCommonStyle {
       return interiorRect(bounds,f).adjusted(t.left,t.top,-t.right,-t.bottom);
     }
 
+    QWidget* getParent(const QWidget *widget, int level) const;
+    bool enoughContrast (const QColor& col1, const QColor& col2) const;
+
     /* Can an expanded border be drawn for this frame? */
     bool hasExpandedBorder(const frame_spec &fspec) const;
 
@@ -334,6 +354,12 @@ class Style : public QCommonStyle {
       forceButtonTextColor(const_cast<QWidget*>(widget), col);
     }
 
+    /* Gets color from #rrggbbaa. */
+    QColor getFromRGBA(const QString &str) const;
+
+    /* Is the window of this widget inactive? */
+    bool isWidgetInactive(const QWidget *widget) const;
+
     /* Used only with combo menus. */
     bool hasParent(const QWidget *widget, const char *className) const
     {
@@ -346,6 +372,12 @@ class Style : public QCommonStyle {
       return false;
     }
 
+    /* Find the kind of this tool-button among a group of tool-buttons (on a toolbar). */
+    int whichGroupedTBtn(const QToolButton *tb, const QWidget *parentBar, bool &drawSeparator) const;
+
+    /* The extra combo box width needed by frames and spacings. */
+    int extraComboWidth(const QStyleOptionComboBox *opt, bool hasIcon) const;
+
 #if QT_VERSION >= 0x050500
     /* For transient scrollbars: */
     void startAnimation(Animation *animation) const;
@@ -419,18 +451,18 @@ class Style : public QCommonStyle {
     bool isKisSlider_;
 
     /* For having clear label icons with QT_DEVICE_PIXEL_RATIO > 1 but without AA_UseHighDpiPixmaps */
-    int pixelRatio_;
-
-    /* For not calculating the extra combo width repeatedly. */
-    mutable int extraComboWidth_;
+    qreal pixelRatio_;
 
     /* Keep track of the sunken button (used instead of a private header for menu positioning). */
     //mutable KvPointer<QWidget> sunkenButton_;
     mutable QPointer<QWidget> sunkenButton_;
 
+    /* Keep track of the widget in which the mouse cursor is entered (to work around Qt scroll jumps). */
+    QPointer<QWidget> enteredWidget_;
+
     /* For not getting the menu shadows repeatedly.
        They're used to position submenus correctly. */
-    QList<int> menuShadow_;
+    QList<int> menuShadow_, realMenuShadow_;
 
     /* Is this DE GTK-based? Currently Gnome, Unity and Pantheon are supported. */
     bool gtkDesktop_;
