@@ -2,18 +2,18 @@
 // and QtCurve's windowmanager.cpp v1.8.17
 
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014 <tsujan2000@gmail.com>
- * 
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014-2019 <tsujan2000@gmail.com>
+ *
  * Kvantum is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * Kvantum is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -36,16 +36,17 @@
 #include <QToolButton>
 #include <QTreeView>
 #include <QGraphicsView>
-#if 0
-#include <QDesktopWidget> // to be used on wayland
-#endif
-
+/* Qt got a new X11 bug since Qt5.11 and, as a result, the X11 drag
+   started to have a problem. However, dragging is still possible by
+   using QWindow::setFramePosition(); hence, these cases of "#if". */
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
+#include <QWindow>
+#include "windowmanager.h"
+#else
 #include "windowmanager.h"
 #include "x11wmmove.h"
+#endif
 
-// WindowManager can be called on wayland only for Qt >= 5.2,
-// in which case, it'll use QWidget::move() for dragging.
-// However, the code is commented out for now.
 namespace Kvantum {
 
 static inline bool isPrimaryToolBar(QWidget *w)
@@ -56,10 +57,13 @@ static inline bool isPrimaryToolBar(QWidget *w)
   {
     if (!tb || Qt::Horizontal == tb->orientation())
     {
+      QWidget *p = w->parentWidget();
+      if (p != w->window()) return false; // inside a dock
+
       if (0 == w->pos().y())
         return true;
 
-      if (QMainWindow *mw = qobject_cast<QMainWindow *>(w->window()))
+      if (QMainWindow *mw = qobject_cast<QMainWindow *>(p))
       {
         if (QWidget *menuW = mw->menuWidget())
           return menuW->isVisible() && w->pos().y() <= menuW->height()+1;
@@ -69,9 +73,9 @@ static inline bool isPrimaryToolBar(QWidget *w)
   return false;
 }
 
-WindowManager::WindowManager (QObject* parent, Drag drag/*, bool isX11*/) :
+WindowManager::WindowManager (QObject* parent, Drag drag) :
                QObject (parent),
-               pixelRatio_ (1),
+               pixelRatio_ (1.0),
                enabled_ (true),
                dragDistance_ (QApplication::startDragDistance()),
                dragDelay_ (QApplication::startDragTime()),
@@ -79,14 +83,13 @@ WindowManager::WindowManager (QObject* parent, Drag drag/*, bool isX11*/) :
                dragInProgress_ (false),
                locked_ (false),
                drag_ (drag)
-#if 0
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
                , cursorOverride_ (false)
 #endif
 {
-  //isX11_ = isX11;
 #if QT_VERSION >= 0x050500
-  int dpr = qApp->devicePixelRatio();
-  if (dpr > 1)
+  qreal dpr = qApp->devicePixelRatio();
+  if (dpr > 1.0)
     pixelRatio_ = dpr;
 #endif
   _appEventFilter = new AppEventFilter( this );
@@ -109,10 +112,7 @@ void WindowManager::registerWidget (QWidget* widget)
     the drag to happen
   */
   if (isBlackListed (widget) || isDragable (widget))
-  {
-    widget->removeEventFilter (this);
     widget->installEventFilter (this);
-  }
 
 }
 /*************************/
@@ -127,10 +127,10 @@ void WindowManager::initializeWhiteList (const QStringList &list)
   whiteList_.clear();
 
   // add user specified whitelisted classnames
-  whiteList_.insert (ExceptionId ("MplayerWindow"));
-  whiteList_.insert (ExceptionId ("Screen@smplayer"));
-  whiteList_.insert (ExceptionId ("ViewSliders@kmix"));
-  whiteList_.insert (ExceptionId ("Sidebar_Widget@konqueror"));
+  whiteList_.insert (ExceptionId (QStringLiteral("MplayerWindow")));
+  whiteList_.insert (ExceptionId (QStringLiteral("Screen@smplayer")));
+  whiteList_.insert (ExceptionId (QStringLiteral("ViewSliders@kmix")));
+  whiteList_.insert (ExceptionId (QStringLiteral("Sidebar_Widget@konqueror")));
 
   for (const QString& exception : list)
   {
@@ -144,8 +144,8 @@ void WindowManager::initializeBlackList (const QStringList &list)
 {
 
   blackList_.clear();
-  blackList_.insert (ExceptionId ("CustomTrackView@kdenlive"));
-  blackList_.insert (ExceptionId ("MuseScore"));
+  blackList_.insert (ExceptionId (QStringLiteral("CustomTrackView@kdenlive")));
+  blackList_.insert (ExceptionId (QStringLiteral("MuseScore")));
   for (const QString& exception : list)
   {
     ExceptionId id (exception);
@@ -218,10 +218,11 @@ bool WindowManager::mousePressEvent (QObject* object, QEvent* event)
 
   // retrieve widget's child at event position
   QPoint position;
-  /*if (!isX11_)
-    position = widget->mapFromGlobal (mouseEvent->globalPos()); // see WindowManager::mouseMoveEvent for the reason
-  else*/
-    position = mouseEvent->pos();
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
+  position = widget->mapFromGlobal (mouseEvent->globalPos()); // see WindowManager::mouseMoveEvent for the reason
+#else
+  position = mouseEvent->pos();
+#endif
 
   QWidget* child = widget->childAt (position);
   if(!canDrag (widget, child, position))
@@ -257,7 +258,7 @@ bool WindowManager::mouseMoveEvent (QObject* object, QEvent* event)
     dragTimer_.stop();
 
   // cast event and check drag distance
-  QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
+  QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(event);
   if (!dragInProgress_)
   {
     if (dragAboutToStart_)
@@ -281,33 +282,21 @@ bool WindowManager::mouseMoveEvent (QObject* object, QEvent* event)
   }
   else
   {
-#if 0
-    if (!isX11_ && target_)
-    { // use QWidget::move for the grabbing
-      QWidget *window (target_.data()->window());
-      QRect ag = QApplication::desktop()->availableGeometry();
-      QRect fg = window->frameGeometry();
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
+    if (target_)
+    { // use QWindow::setFramePosition (or QWidget::move) for the dragging
+      QWidget* window = target_.data()->window();
       /* FIXME: For some unknown reason, mapping from the global position
          results in smoother movements than mouseEvent->pos() does,
          especially when the window is translucent. */
       QPoint pos = target_.data()->mapFromGlobal (mouseEvent->globalPos());
-      QPoint winPos = window->pos();
-      QPoint newWinPos = winPos + pos - dragPoint_;
-      /* if the window is completely inside the screen, don't try to
-         move it offscreen because some WM's (like KWin) don't allow that */
-      if (ag.contains (fg))
-      {
-        if (newWinPos.x() < ag.x())
-          pos = QPoint (ag.x() + (dragPoint_ - winPos).x(), pos.y());
-        else if (newWinPos.x() + fg.width() > ag.x() + ag.width())
-          pos = QPoint (ag.x() + ag.width() - fg.width() + (dragPoint_ - winPos).x(), pos.y());
-        if (newWinPos.y() < ag.y())
-          pos = QPoint (pos.x(), ag.y() + (dragPoint_ - winPos).y());
-        else if (newWinPos.y() + fg.height() > ag.y() + ag.height())
-          pos = QPoint (pos.x(), ag.y() + ag.height() - fg.height() + (dragPoint_ - winPos).y());
-      }
-
-      window->move (winPos + pos - dragPoint_);
+      /* FIXME: Again, for some unknown reason, QWidget::move() could result
+         in a choppy movement with some apps, while QWindow::setFramePosition()
+         is always smooth, provided that the window is native. */
+      if (QWindow* w = window->windowHandle())
+        w->setFramePosition (w->framePosition() + pos - dragPoint_);
+      else
+        window->move (window->pos() + pos - dragPoint_); // should not happen
       return true;
     }
 #endif
@@ -383,7 +372,7 @@ bool WindowManager::isDragable (QWidget* widget)
   /*
     catch labels in status bars.
     this is because of kstatusbar
-    who captures buttonPress/release events
+    that captures buttonPress/release events
   */
   if (QLabel* label = qobject_cast<QLabel*>(widget))
   {
@@ -463,19 +452,16 @@ bool WindowManager::canDrag (QWidget* widget)
   if (widget->cursor().shape() != Qt::ArrowCursor)
     return false;
 
-#if 0
-  if (!isX11_)
-  {
-    // X11BypassWindowManagerHint can be used to have fixed position
-    if (widget->window()->windowFlags().testFlag(Qt::X11BypassWindowManagerHint))
-      return false;
-    /* Qt5 has a weird "hover bug", because of which, the cursor may not
-       be over the widget but mousePressEvent() may be called on pressing
-       the left mouse button. Since this function is called first by
-       WindowManager::mousePressEvent(), we add a workaround here. */
-    if (!widget->rect().contains(widget->mapFromGlobal(QCursor::pos())))
-      return false;
-  }
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
+  // X11BypassWindowManagerHint can be used to have fixed position
+  if (widget->window()->windowFlags().testFlag(Qt::X11BypassWindowManagerHint))
+    return false;
+  /* Qt5 has a weird "hover bug", because of which, the cursor may not
+     be over the widget but mousePressEvent() may be called on pressing
+     the left mouse button. Since this function is called first by
+     WindowManager::mousePressEvent(), we add a workaround here. */
+  if (!widget->rect().contains(widget->mapFromGlobal(QCursor::pos())))
+    return false;
 #endif
 
   // accept
@@ -595,9 +581,9 @@ bool WindowManager::canDrag (QWidget* widget, QWidget* child, const QPoint& posi
   }
 
   // abstract item views
-  QAbstractItemView* itemView (NULL);
-  if ((itemView = qobject_cast<QListView*>( widget->parentWidget()))
-      || (itemView = qobject_cast<QTreeView*>( widget->parentWidget())))
+  QAbstractItemView* itemView (nullptr);
+  if ((itemView = qobject_cast<QListView*>(widget->parentWidget()))
+      || (itemView = qobject_cast<QTreeView*>(widget->parentWidget())))
   {
     if (widget == itemView->viewport())
     {
@@ -645,10 +631,10 @@ bool WindowManager::canDrag (QWidget* widget, QWidget* child, const QPoint& posi
   return true;
 }
 /*************************/
-void WindowManager::resetDrag (void)
+void WindowManager::resetDrag()
 {
-#if 0
-  if (!isX11_ && target_ && cursorOverride_)
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
+  if (target_ && cursorOverride_)
   {
     qApp->restoreOverrideCursor();
     cursorOverride_ = false;
@@ -666,19 +652,25 @@ void WindowManager::resetDrag (void)
 /*************************/
 void WindowManager::startDrag (QWidget *widget, const QPoint &position)
 {
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
+  Q_UNUSED(position);
+#endif
+
   if (!(enabled() && widget) || QWidget::mouseGrabber())
     return;
 
-  //if (isX11_)
-  //{
-    X11MoveTrigger (widget->window()->internalWinId(),
-                    position.x()*pixelRatio_, position.y()*pixelRatio_);
-  /*}
-  else if (!cursorOverride_)
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
+  if (!cursorOverride_)
   {
     qApp->setOverrideCursor (Qt::DragMoveCursor);
     cursorOverride_ = true;
-  }*/
+  }
+#else
+  X11MoveTrigger (widget->window()->internalWinId(),
+                  qRound(static_cast<qreal>(position.x())*pixelRatio_),
+                  qRound(static_cast<qreal>(position.y())*pixelRatio_));
+#endif
 
   dragInProgress_ = true;
 }
@@ -695,6 +687,10 @@ bool WindowManager::isDockWidgetTitle (const QWidget* widget) const
 /*************************/
 bool WindowManager::AppEventFilter::eventFilter (QObject* object, QEvent* event)
 {
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
+  Q_UNUSED(object);
+#endif
+
   if (event->type() == QEvent::MouseButtonRelease)
   {
     // stop drag timer
@@ -706,25 +702,26 @@ bool WindowManager::AppEventFilter::eventFilter (QObject* object, QEvent* event)
       parent_->setLocked (false);
   }
 
-  //if (parent_->isX11_)
-  //{
-    if (!parent_->enabled()) return false;
+#if (QT_VERSION >= QT_VERSION_CHECK(5,11,0))
+  return false;
+#else
+  if (!parent_->enabled()) return false;
 
-    /*
-      If a drag is in progress, the widget will not receive any event.
-      We trigger on the first MouseMove or MousePress events that are received
-      by any widget in the application to detect that the drag is finished.
-    */
-    if (parent_->dragInProgress_
-        && parent_->target_
-        && (event->type() == QEvent::MouseMove
-            || event->type() == QEvent::MouseButtonPress))
-    {
-      return appMouseEvent (object, event);
-    }
-  //}
+  /*
+    If a drag is in progress, the widget will not receive any event.
+    We trigger on the first MouseMove or MousePress events that are received
+    by any widget in the application to detect that the drag is finished.
+  */
+  if (parent_->dragInProgress_
+      && parent_->target_
+      && (event->type() == QEvent::MouseMove
+          || event->type() == QEvent::MouseButtonPress))
+  {
+    return appMouseEvent (object, event);
+  }
 
   return false;
+#endif
 }
 /*************************/
 bool WindowManager::AppEventFilter::appMouseEvent (QObject* object, QEvent* event)
