@@ -93,6 +93,8 @@ typedef struct {
   bool spread_progressbar;
   /* thin progressbars? */
   int progressbar_thickness;
+  /* spread a header so that its interior meets its view frame if possible? */
+  bool spread_header;
   /* enable mouse tracking in menubars? */
   bool menubar_mouse_tracking;
   /* use composite effects
@@ -104,19 +106,28 @@ typedef struct {
   /* reduce window opacity by this percentage
      (if window translucency is enabled)? */
   int reduce_window_opacity;
+  /* the same as above but for menus */
+  int reduce_menu_opacity;
   /* list of apps that shouldn't have window translucency */
   QStringList opaque;
   /* blur what is behind translucent windows if possible? */
   bool blurring;
   /* blur what is behind menus/tooltips if possible? */
   bool popup_blurring;
+  /* values needed for the KDE contrast effect: */
+  qreal contrast, intensity, saturation;
   /* should buttons and comboboxes be animated
      under the cursor when their state changes? */
   bool animate_states;
+  /* disable inactiveness */
+  bool no_inactiveness;
   /* disable window/dialog patterns */
   bool no_window_pattern;
   /* depth of menu shadows */
   int menu_shadow_depth;
+  int menu_separator_height;
+  /* should menuitems spread across the whole menu horizontally? */
+  bool spread_menuitems;
   /* overlap between a submenu and its parent */
   int submenu_overlap;
   /* milliseconds to wait before opening a submenu
@@ -130,6 +141,8 @@ typedef struct {
   int scroll_width;
   /* minimum scrollbar extent */
   int scroll_min_extent;
+  /* should the scrollbar indicator (grip) be centered? */
+  bool center_scrollbar_indicator;
   /* draw scroll arrows? */
   bool scroll_arrows;
   /* draw tree branch lines? */
@@ -185,6 +198,8 @@ typedef struct {
   bool transient_groove;
   /* should we request a dark titlebar under Gtk desktops? */
   bool dark_titlebar;
+  /* the layout of dialog buttons */
+  int dialog_button_layout;
 
   int layout_spacing;
   int layout_margin;
@@ -202,6 +217,7 @@ typedef struct {
   QString baseColor;
   QString inactiveBaseColor;
   QString altBaseColor;
+  QString inactiveAltBaseColor;
   QString buttonColor;
   QString lightColor;
   QString midLightColor;
@@ -240,6 +256,8 @@ typedef struct {
   /* blur the region behind a window background that is
      explicitly made translucent by its app? */
   bool blur_translucent;
+  /* should all texts have opaque colors? */
+  bool opaque_colors;
   /* transparent background for the label of KTitleWidget
      (nice when the window bg has a gradient) */
   bool transparent_ktitle_label;
@@ -273,6 +291,8 @@ typedef struct {
   /* should the scroll jump be done by middle clicking?
      (By default, it's done by left clicking.) */
   bool middle_click_scroll;
+  /* workaround for annoying Qt5 scroll jumps? */
+  bool scroll_jump_workaround;
 } hacks_spec;
 
 /* Generic information about a frame */
@@ -463,22 +483,31 @@ static inline void default_theme_spec(theme_spec &tspec) {
   tspec.toolbutton_style = 0;
   tspec.spread_progressbar = false;
   tspec.progressbar_thickness = 0;
+  tspec.spread_header = false;
   tspec.menubar_mouse_tracking = true;
   tspec.composite = false;
   tspec.translucent_windows = false;
   tspec.reduce_window_opacity = 0;
+  tspec.reduce_menu_opacity = 0;
   tspec.opaque = QStringList() << "kscreenlocker" << "wine";
   tspec.blurring = false;
   tspec.popup_blurring = false;
+  tspec.contrast = static_cast<qreal>(1);
+  tspec.intensity = static_cast<qreal>(1);
+  tspec.saturation = static_cast<qreal>(1);
   tspec.animate_states = false;
+  tspec.no_inactiveness = false;
   tspec.no_window_pattern = false;
   tspec.menu_shadow_depth = 0;
+  tspec.menu_separator_height = 10;
+  tspec.spread_menuitems = false;
   tspec.submenu_overlap = 0;
   tspec.submenu_delay = 250;
   tspec.tooltip_shadow_depth = 0;
   tspec.splitter_width = 7;
   tspec.scroll_width = 12;
   tspec.scroll_min_extent = 36;
+  tspec.center_scrollbar_indicator = false;
   tspec.scroll_arrows=true;
   tspec.tree_branch_line=false;
   tspec.slider_width = 8;
@@ -495,7 +524,7 @@ static inline void default_theme_spec(theme_spec &tspec) {
   tspec.combo_menu = false;
   tspec.hide_combo_checkboxes = false;
   tspec.combo_focus_rect = false;
-  tspec.scrollable_menu = false;
+  tspec.scrollable_menu = true;
   tspec.fill_rubberband = false;
   tspec.groupbox_top_label = false;
   tspec.button_contents_shift = true;
@@ -503,6 +532,7 @@ static inline void default_theme_spec(theme_spec &tspec) {
   tspec.transient_scrollbar = false;
   tspec.transient_groove = false;
   tspec.dark_titlebar = false;
+  tspec.dialog_button_layout = 0;
   tspec.layout_spacing = 2;
   tspec.layout_margin = 4;
   tspec.small_icon_size = 16;
@@ -517,6 +547,7 @@ static inline void default_color_spec(color_spec &cspec) {
   cspec.baseColor = QString();
   cspec.inactiveBaseColor = QString();
   cspec.altBaseColor = QString();
+  cspec.inactiveAltBaseColor = QString();
   cspec.buttonColor = QString();
   cspec.lightColor = QString();
   cspec.midLightColor = QString();
@@ -547,6 +578,7 @@ static inline void default_hacks_spec(hacks_spec &hspec) {
   hspec.transparent_pcmanfm_view = false;
   hspec.lxqtmainmenu_iconsize = 0;
   hspec.blur_translucent = false;
+  hspec.opaque_colors = false;
   hspec.transparent_ktitle_label = false;
   hspec.transparent_menutitle = false;
   hspec.kcapacitybar_as_progressbar = false;
@@ -561,6 +593,7 @@ static inline void default_hacks_spec(hacks_spec &hspec) {
   hspec.iconless_menu = false;
   hspec.single_top_toolbar = false;
   hspec.middle_click_scroll = false;
+  hspec.scroll_jump_workaround = false;
 }
 }
 
